/**
 * connection.h (2007-04-12)
 *
 * -- CABAL -- connection header
 *
 * Copyright (c) 2007, NOD Team. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *	* Redistributions of source code must retain the above copyright
 *	  notice, this list of conditions and the following disclaimer.
 *	* Redistributions in binary form must reproduce the above copyright
 *	  notice, this list of conditions and the following disclaimer in the
 *	  documentation and/or other materials provided with the distribution.
 *	* Neither the name of the NOD Team nor the names of its contributors
 *	  may be used to endorse or promote products derived from this
 *	  software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _CONNECTION_H_
#define _CONNECTION_H_

#include "network.h"

class Connection : public Network {

public:
	Connection();
	~Connection();

	void ircConnect();

	void ircPass (string);
	void ircNick (string);
	void ircUser (string, string);
	void ircAway (string);
	void ircJoin (string, string);
	void ircPart (string);
	void ircCycle (string);

	void establish();

	struct host_type {
		pchar host;
		int port;
		pchar password;
		host_type *next;

		host_type (pchar h, int p, pchar pw)
			: host (h), port (p), password (pw), next (this) {};
		host_type (pchar h, int p, pchar pw, host_type *t)
			: host (h), port (p), password (pw), next (t) {};
	} *hosts, *current;

	host_type *hostAdd (host_type *, pchar, int, pchar);

	pchar virtualhost, nick, user, name, chan, ckey;

	time_t now, uptime;
};

#endif // _CONNECTION_H_
